import csv
import os
import pandas as pd
import matplotlib.pyplot as plt
from datetime import datetime

# Load questions from the CSV file
def load_questions():
    questions = []
    if os.path.exists("questions.csv"):
        with open("questions.csv", "r") as file:
            reader = csv.reader(file)
            for row in reader:
                questions.append(row)
    return questions

# Add a new question to the CSV file
def add_question():
    question = input("Enter the question: ")
    options = [input(f"Enter option {i+1}: ") for i in range(4)]
    correct_answer = input("Enter the correct answer (1/2/3/4): ")
    with open("questions.csv", "a", newline="") as file:
        writer = csv.writer(file)
        writer.writerow([question, *options, correct_answer])
    print("Question added successfully!")

# Start the quiz
def start_quiz():
    questions = load_questions()
    if not questions:
        print("No questions available!")
        return
    
    score = 0
    total_questions = len(questions)
    print("Starting the quiz...\n")
    for i, question in enumerate(questions):
        print(f"Q{i+1}: {question[0]}")
        for j, option in enumerate(question[1:5]):
            print(f"{j+1}. {option}")
        answer = input("Your answer (1/2/3/4): ")
        if answer == question[5]:
            score += 1
            print("Correct!\n")
        else:
            print(f"Wrong! The correct answer is {question[5]}\n")
    
    print(f"Your total score: {score}/{total_questions}")
    save_score(score)
    
    # Display Pie Chart
    quiz_performance_pie(score, total_questions)

# Save user score to the CSV file
def save_score(score):
    name = input("Enter your name: ")
    with open("scores.csv", "a", newline="") as file:
        writer = csv.writer(file)
        writer.writerow([name, score, datetime.now().strftime("%Y-%m-%d %H:%M:%S")])
    print("Score saved successfully!")

# Visualize scores with a bar chart
def visualize_scores():
    if not os.path.exists("scores.csv"):
        print("No scores available for visualization!")
        return
    
    df = pd.read_csv("scores.csv", names=["Name", "Score", "Date"])
    grouped = df.groupby("Name")["Score"].sum()
    grouped.plot(kind="bar", color="skyblue", figsize=(8, 6))
    plt.title("Total Scores by Participants")
    plt.xlabel("Participants")
    plt.ylabel("Total Score")
    plt.xticks(rotation=45)
    plt.tight_layout()
    plt.show()

# Generate a pie chart for quiz performance
def quiz_performance_pie(correct, total):
    incorrect = total - correct
    labels = ['Correct', 'Incorrect']
    sizes = [correct, incorrect]
    colors = ['green', 'red']
    
    plt.figure(figsize=(6, 6))
    plt.pie(sizes, labels=labels, colors=colors, autopct='%1.1f%%', startangle=140)
    plt.title("Quiz Performance Breakdown")
    plt.axis('equal')
    plt.show()

# Track a user's progress with a line graph
def track_user_progress(name):
    if not os.path.exists("scores.csv"):
        print("No scores available!")
        return
    
    df = pd.read_csv("scores.csv", names=["Name", "Score", "Date"])
    user_data = df[df["Name"] == name]
    
    if user_data.empty:
        print(f"No data available for user: {name}")
        return
    
    user_data["Date"] = pd.to_datetime(user_data["Date"])
    user_data = user_data.sort_values(by="Date")
    
    plt.figure(figsize=(8, 6))
    plt.plot(user_data["Date"], user_data["Score"], marker="o", linestyle="-", color="blue")
    plt.title(f"Score Progress for {name}")
    plt.xlabel("Date")
    plt.ylabel("Score")
    plt.xticks(rotation=45)
    plt.tight_layout()
    plt.show()

# Admin menu
def admin_menu():
    while True:
        print("\nAdmin Menu:")
        print("1. Add Question")
        print("2. View Questions")
        print("3. Visualize Scores (Bar Chart)")
        print("4. Exit")
        choice = input("Enter your choice: ")
        
        if choice == "1":
            add_question()
        elif choice == "2":
            questions = load_questions()
            for question in questions:
                print(question)
        elif choice == "3":
            visualize_scores()
        elif choice == "4":
            break
        else:
            print("Invalid choice! Try again.")

# Main function
def main():
    while True:
        print("\nMain Menu:")
        print("1. Start Quiz")
        print("2. Admin Menu")
        print("3. Track User Progress")
        print("4. Exit")
        choice = input("Enter your choice: ")
        
        if choice == "1":
            start_quiz()
        elif choice == "2":
            admin_menu()
        elif choice == "3":
            name = input("Enter your name to track progress: ")
            track_user_progress(name)
        elif choice == "4":
            print("Exiting the application. Goodbye!")
            break
        else:
            print("Invalid choice! Try again.")

if __name__ == "__main__":
    main()
